//+------------------------------------------------------------------+
//|                                                Zero lag MACD.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Lime//DeepSkyBlue
#property indicator_color2 DimGray
#property indicator_color3 Lime//DeepSkyBlue
#property indicator_color4 Red
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2
//
//
//
//
//

extern string timeFrame      = "current time frame";
extern int    FastEMA        =  8;//12
extern int    SlowEMA        = 15;//26
extern int    SignalEMA      =  6;///9
extern int    MacdPrice      = PRICE_CLOSE;
extern bool   Interpolate    = true;
//
//
//
//
//

double macdBuffer[];
double machBuffer[];
double signBuffer[];
double osmaBuffer[];
double workBuffer[][6];

//
//
//
//
//

string IndicatorFileName;
bool   calculatingMacd = false;
bool   returningBars   = false;
int    TimeFrame;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,machBuffer); SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,NULL);
   SetIndexBuffer(1,osmaBuffer); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,macdBuffer); 
   SetIndexBuffer(3,signBuffer);

   //
   //
   //
   //
   //
   
   if (timeFrame=="calculateMACD") { calculatingMacd=true; return(0); }
   if (timeFrame=="returnBars")    { returningBars=true;   return(0); }

   //
   //
   //
   //
   //
   //

   TimeFrame = stringToTimeFrame(timeFrame);
   
   string TimeFrameStr;
   switch(TimeFrame)
   {
      case PERIOD_M1:  TimeFrameStr="(M1)";      break;
      case PERIOD_M5:  TimeFrameStr="(M5)";      break;
      case PERIOD_M15: TimeFrameStr="(M15)";     break;
      case PERIOD_M30: TimeFrameStr="(M30)";     break;
      case PERIOD_H1:  TimeFrameStr="(H1)";      break;
      case PERIOD_H4:  TimeFrameStr="(H4)";      break;
      case PERIOD_D1:  TimeFrameStr="(Dayly)";   break;
      case PERIOD_W1:  TimeFrameStr="(Weekly)";  break;
      case PERIOD_MN1: TimeFrameStr="(Monthly)"; break;
      default :        TimeFrameStr="";
   }
   IndicatorFileName = WindowExpertName();               
   IndicatorShortName(" zero lag MACD ("+FastEMA+","+SlowEMA+","+SignalEMA+")"+TimeFrameStr);
   return(0);
}
int deinit(){ return(0); }




//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
   int limit,i;

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);

           if (returningBars)   { macdBuffer[0] = limit; return(0); }
           if (calculatingMacd) { CalculateMacd(limit);  return(0); }
           if (TimeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,TimeFrame,IndicatorFileName,"returnBars",0,0)*TimeFrame/Period()));

   //
   //
   //
   //
   //
   
 	for(i = limit; i >= 0; i--)
   {
      int      shift1 = iBarShift(NULL,TimeFrame,Time[i]);
      datetime time1  = iTime    (NULL,TimeFrame,shift1);

      //
      //
      //
      //
      //
               
         macdBuffer[i] = iCustom(NULL,TimeFrame,IndicatorFileName,"calculateMACD",FastEMA,SlowEMA,SignalEMA,MacdPrice,2,shift1);
         signBuffer[i] = iCustom(NULL,TimeFrame,IndicatorFileName,"calculateMACD",FastEMA,SlowEMA,SignalEMA,MacdPrice,3,shift1);
         machBuffer[i] = macdBuffer[i];
         osmaBuffer[i] = macdBuffer[i]-signBuffer[i];

         if (TimeFrame <= Period() || shift1==iBarShift(NULL,TimeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;		 

      //
      //
      //
      //
      //

         for(int n = 1; i+n < Bars && Time[i+n] >= time1; n++) continue;	
         double factor = 1.0 / n;
         for(int k = 1; k < n; k++)
            {
               macdBuffer[i+k] = k*factor*macdBuffer[i+n] + (1.0-k*factor)*macdBuffer[i];
    	         signBuffer[i+k] = k*factor*signBuffer[i+n] + (1.0-k*factor)*signBuffer[i];
    	         machBuffer[i+k] = macdBuffer[i+k];
    	         osmaBuffer[i+k] = macdBuffer[i+k]-signBuffer[i+k];
            }    	             
   }
   return(0);           
}



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define ema11 0
#define ema12 1
#define ema21 2
#define ema22 3
#define ema31 4
#define ema32 5

//
//
//
//
//

void CalculateMacd(int limit)
{
   double alpha1 = 2.0/(1.0+FastEMA);
   double alpha2 = 2.0/(1.0+SlowEMA);
   double alpha3 = 2.0/(1.0+SignalEMA);
   int i,r;

      if (ArrayRange(workBuffer,0)!=Bars) ArrayResize(workBuffer,Bars);
      
   //
   //
   //
   //
   //
         
   for (i=limit, r=Bars-i-1; i>= 0; i--,r++)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,MacdPrice,i);
      if (i==(Bars-1))
      {
         workBuffer[r][ema11] = price;
         workBuffer[r][ema12] = price;
         workBuffer[r][ema21] = price;
         workBuffer[r][ema22] = price;
         continue;
      }
      workBuffer[r][ema11] = workBuffer[r-1][ema11]+alpha1*(price               -workBuffer[r-1][ema11]);
      workBuffer[r][ema12] = workBuffer[r-1][ema12]+alpha1*(workBuffer[r][ema11]-workBuffer[r-1][ema12]);
      workBuffer[r][ema21] = workBuffer[r-1][ema21]+alpha2*(price               -workBuffer[r-1][ema21]);
      workBuffer[r][ema22] = workBuffer[r-1][ema22]+alpha2*(workBuffer[r][ema21]-workBuffer[r-1][ema22]);
      macdBuffer[i]        = (2.0*workBuffer[r][ema11]-workBuffer[r][ema12])-
                             (2.0*workBuffer[r][ema21]-workBuffer[r][ema22]);
      machBuffer[i]        = macdBuffer[i];                      
 
      //
      //
      //
      //
      //
      
      workBuffer[r][ema31] = workBuffer[r-1][ema31]+alpha3*(macdBuffer[i]       -workBuffer[r-1][ema31]);
      workBuffer[r][ema32] = workBuffer[r-1][ema32]+alpha3*(workBuffer[r][ema31]-workBuffer[r-1][ema32]);
      signBuffer[i]        = (2.0*workBuffer[r][ema31]-workBuffer[r][ema32]);
      osmaBuffer[i]        = macdBuffer[i]-signBuffer[i];            
   }

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   for(int l = StringLen(tfs)-1; l >= 0; l--)
   {
      int char = StringGetChar(tfs,l);
          if((char > 96 && char < 123) || (char > 223 && char < 256))
               tfs = StringSetChar(tfs, l, char - 32);
          else 
              if(char > -33 && char < 0)
                  tfs = StringSetChar(tfs, l, char + 224);
   }

   //
   //
   //
   //
   //
   
   int tf=0;
         if (tfs=="M1" || tfs=="1")     tf=PERIOD_M1;
         if (tfs=="M5" || tfs=="5")     tf=PERIOD_M5;
         if (tfs=="M15"|| tfs=="15")    tf=PERIOD_M15;
         if (tfs=="M30"|| tfs=="30")    tf=PERIOD_M30;
         if (tfs=="H1" || tfs=="60")    tf=PERIOD_H1;
         if (tfs=="H4" || tfs=="240")   tf=PERIOD_H4;
         if (tfs=="D1" || tfs=="1440")  tf=PERIOD_D1;
         if (tfs=="W1" || tfs=="10080") tf=PERIOD_W1;
         if (tfs=="MN" || tfs=="43200") tf=PERIOD_MN1;
   return(tf);
}